@@ -32,13 +32,14 @@ from gitlab.exceptions import *  # noqa
 from gitlab.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '0.12.2'
+__version__ = '0.14'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
 __copyright__ = 'Copyright 2013-2016 Gauvain Pocentek'
 
-warnings.simplefilter('always', DeprecationWarning)
+warnings.filterwarnings('default', category=DeprecationWarning,
+                        module='^gitlab')
 
 
 def _sanitize(value):
@@ -61,19 +62,34 @@ class Gitlab(object):
         ssl_verify (bool): Whether SSL certificates should be validated.
         timeout (float or tuple(float,float)): Timeout to use for requests to
             the GitLab server.
-
+        http_username: (str): Username for HTTP authentication
+        http_password: (str): Password for HTTP authentication
     Attributes:
+        user_emails (UserEmailManager): Manager for GitLab users' emails.
         user_keys (UserKeyManager): Manager for GitLab users' SSH keys.
         users (UserManager): Manager for GitLab users
+        keys (DeployKeyManager): Manager for deploy keys
+        group_issues (GroupIssueManager): Manager for GitLab group issues
+        group_projects (GroupProjectManager): Manager for GitLab group projects
         group_members (GroupMemberManager): Manager for GitLab group members
         groups (GroupManager): Manager for GitLab members
         hooks (HookManager): Manager for GitLab hooks
         issues (IssueManager): Manager for GitLab issues
+        licenses (LicenseManager): Manager for licenses
+        namespaces (NamespaceManager): Manager for namespaces
         project_branches (ProjectBranchManager): Manager for GitLab projects
             branches
+        project_builds (ProjectBuildManager): Manager for GitLab projects
+            builds
         project_commits (ProjectCommitManager): Manager for GitLab projects
             commits
+        project_commit_comments (ProjectCommitCommentManager): Manager for
+            GitLab projects commits comments
+        project_commit_statuses (ProjectCommitStatusManager): Manager for
+            GitLab projects commits statuses
         project_keys (ProjectKeyManager): Manager for GitLab projects keys
+        project_environments (ProjectEnvironmentManager): Manager for GitLab
+            projects environments
         project_events (ProjectEventManager): Manager for GitLab projects
             events
         project_forks (ProjectForkManager): Manager for GitLab projects forks
@@ -99,15 +115,20 @@ class Gitlab(object):
             note on snippets
         project_snippets (ProjectSnippetManager): Manager for GitLab projects
             snippets
+        project_triggers (ProjectTriggerManager): Manager for build triggers
+        project_variables (ProjectVariableManager): Manager for build variables
         user_projects (UserProjectManager): Manager for GitLab projects users
         projects (ProjectManager): Manager for GitLab projects
+        runners (RunnerManager): Manager for the CI runners
+        settings (ApplicationSettingsManager): manager for the Gitlab settings
         team_members (TeamMemberManager): Manager for GitLab teams members
         team_projects (TeamProjectManager): Manager for GitLab teams projects
         teams (TeamManager): Manager for GitLab teams
     """
 
-    def __init__(self, url, private_token=None,
-                 email=None, password=None, ssl_verify=True, timeout=None):
+    def __init__(self, url, private_token=None, email=None, password=None,
+                 ssl_verify=True, http_username=None, http_password=None,
+                 timeout=None):
 
         self._url = '%s/api/v3' % url
         #: Timeout to use for requests to gitlab server
@@ -121,17 +142,32 @@ class Gitlab(object):
         self.password = password
         #: Whether SSL certificates should be validated
         self.ssl_verify = ssl_verify
+        self.http_username = http_username
+        self.http_password = http_password
+
+        #: Create a session object for requests
+        self.session = requests.Session()
 
         self.settings = ApplicationSettingsManager(self)
+        self.user_emails = UserEmailManager(self)
         self.user_keys = UserKeyManager(self)
         self.users = UserManager(self)
+        self.keys = KeyManager(self)
+        self.group_issues = GroupIssueManager(self)
+        self.group_projects = GroupProjectManager(self)
         self.group_members = GroupMemberManager(self)
         self.groups = GroupManager(self)
         self.hooks = HookManager(self)
         self.issues = IssueManager(self)
+        self.licenses = LicenseManager(self)
+        self.namespaces = NamespaceManager(self)
         self.project_branches = ProjectBranchManager(self)
+        self.project_builds = ProjectBuildManager(self)
         self.project_commits = ProjectCommitManager(self)
+        self.project_commit_comments = ProjectCommitCommentManager(self)
+        self.project_commit_statuses = ProjectCommitStatusManager(self)
         self.project_keys = ProjectKeyManager(self)
+        self.project_environments = ProjectEnvironmentManager(self)
         self.project_events = ProjectEventManager(self)
         self.project_forks = ProjectForkManager(self)
         self.project_hooks = ProjectHookManager(self)
@@ -147,8 +183,11 @@ class Gitlab(object):
         self.project_files = ProjectFileManager(self)
         self.project_snippet_notes = ProjectSnippetNoteManager(self)
         self.project_snippets = ProjectSnippetManager(self)
+        self.project_triggers = ProjectTriggerManager(self)
+        self.project_variables = ProjectVariableManager(self)
         self.user_projects = UserProjectManager(self)
         self.projects = ProjectManager(self)
+        self.runners = RunnerManager(self)
         self.team_members = TeamMemberManager(self)
         self.team_projects = TeamProjectManager(self)
         self.teams = TeamManager(self)
@@ -170,7 +209,9 @@ class Gitlab(object):
         config = gitlab.config.GitlabConfigParser(gitlab_id=gitlab_id,
                                                   config_files=config_files)
         return Gitlab(config.url, private_token=config.token,
-                      ssl_verify=config.ssl_verify, timeout=config.timeout)
+                      ssl_verify=config.ssl_verify, timeout=config.timeout,
+                      http_username=config.http_username,
+                      http_password=config.http_password)
 
     def auth(self):
         """Performs an authentication.
@@ -211,14 +252,24 @@ class Gitlab(object):
         """
         self._url = '%s/api/v3' % url
 
-    def _construct_url(self, id_, obj, parameters):
+    def _construct_url(self, id_, obj, parameters, action=None):
         if 'next_url' in parameters:
             return parameters['next_url']
         args = _sanitize(parameters)
+
+        url_attr = '_url'
+        if action is not None:
+            attr = '_%s_url' % action
+            if hasattr(obj, attr):
+                url_attr = attr
+        obj_url = getattr(obj, url_attr)
+
+        # TODO(gpocentek): the following will need an update when we have
+        # object with both urlPlural and _ACTION_url attributes
         if id_ is None and obj._urlPlural is not None:
             url = obj._urlPlural % args
         else:
-            url = obj._url % args
+            url = obj_url % args
 
         if id_ is not None:
             url = '%s%s/%s' % (self._url, url, str(id_))
@@ -255,25 +306,29 @@ class Gitlab(object):
         self.email = email
         self.password = password
 
-    def _raw_get(self, path, content_type=None, **kwargs):
+    def _raw_get(self, path, content_type=None, streamed=False, **kwargs):
         url = '%s%s' % (self._url, path)
         headers = self._create_headers(content_type)
-
         try:
-            return requests.get(url,
-                                params=kwargs,
-                                headers=headers,
-                                verify=self.ssl_verify,
-                                timeout=self.timeout)
+            return self.session.get(url,
+                                    params=kwargs,
+                                    headers=headers,
+                                    verify=self.ssl_verify,
+                                    timeout=self.timeout,
+                                    stream=streamed,
+                                    auth=requests.auth.HTTPBasicAuth(
+                                        self.http_username,
+                                        self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
 
-    def _raw_list(self, path, cls, **kwargs):
+    def _raw_list(self, path, cls, extra_attrs={}, **kwargs):
         r = self._raw_get(path, **kwargs)
         raise_error_from_response(r, GitlabListError)
 
-        cls_kwargs = kwargs.copy()
+        cls_kwargs = extra_attrs.copy()
+        cls_kwargs.update(kwargs.copy())
 
         # Add _from_api manually, because we are not creating objects
         # through normal path
@@ -281,7 +336,7 @@ class Gitlab(object):
         get_all_results = kwargs.get('all', False)
 
         # Remove parameters from kwargs before passing it to constructor
-        for key in ['all', 'page', 'per_page', 'sudo']:
+        for key in ['all', 'page', 'per_page', 'sudo', 'next_url']:
             if key in cls_kwargs:
                 del cls_kwargs[key]
 
@@ -298,10 +353,13 @@ class Gitlab(object):
         url = '%s%s' % (self._url, path)
         headers = self._create_headers(content_type)
         try:
-            return requests.post(url, params=kwargs, data=data,
-                                 headers=headers,
-                                 verify=self.ssl_verify,
-                                 timeout=self.timeout)
+            return self.session.post(url, params=kwargs, data=data,
+                                     headers=headers,
+                                     verify=self.ssl_verify,
+                                     timeout=self.timeout,
+                                     auth=requests.auth.HTTPBasicAuth(
+                                         self.http_username,
+                                         self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -311,10 +369,13 @@ class Gitlab(object):
         headers = self._create_headers(content_type)
 
         try:
-            return requests.put(url, data=data, params=kwargs,
-                                headers=headers,
-                                verify=self.ssl_verify,
-                                timeout=self.timeout)
+            return self.session.put(url, data=data, params=kwargs,
+                                    headers=headers,
+                                    verify=self.ssl_verify,
+                                    timeout=self.timeout,
+                                    auth=requests.auth.HTTPBasicAuth(
+                                        self.http_username,
+                                        self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -324,11 +385,14 @@ class Gitlab(object):
         headers = self._create_headers(content_type)
 
         try:
-            return requests.delete(url,
-                                   params=kwargs,
-                                   headers=headers,
-                                   verify=self.ssl_verify,
-                                   timeout=self.timeout)
+            return self.session.delete(url,
+                                       params=kwargs,
+                                       headers=headers,
+                                       verify=self.ssl_verify,
+                                       timeout=self.timeout,
+                                       auth=requests.auth.HTTPBasicAuth(
+                                           self.http_username,
+                                           self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -365,10 +429,16 @@ class Gitlab(object):
         for attribute in obj_class.requiredUrlAttrs:
             del params[attribute]
 
+        # Also remove the next-url attribute that make queries fail
+        if 'next_url' in params:
+            del params['next_url']
         try:
-            r = requests.get(url, params=params, headers=headers,
-                             verify=self.ssl_verify,
-                             timeout=self.timeout)
+            r = self.session.get(url, params=params, headers=headers,
+                                 verify=self.ssl_verify,
+                                 timeout=self.timeout,
+                                 auth=requests.auth.HTTPBasicAuth(
+                                     self.http_username,
+                                     self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -385,7 +455,7 @@ class Gitlab(object):
         get_all_results = params.get('all', False)
 
         # Remove parameters from kwargs before passing it to constructor
-        for key in ['all', 'page', 'per_page', 'sudo']:
+        for key in ['all', 'page', 'per_page', 'sudo', 'next_url']:
             if key in cls_kwargs:
                 del cls_kwargs[key]
 
@@ -434,8 +504,11 @@ class Gitlab(object):
             del params[attribute]
 
         try:
-            r = requests.get(url, params=params, headers=headers,
-                             verify=self.ssl_verify, timeout=self.timeout)
+            r = self.session.get(url, params=params, headers=headers,
+                                 verify=self.ssl_verify, timeout=self.timeout,
+                                 auth=requests.auth.HTTPBasicAuth(
+                                     self.http_username,
+                                     self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -463,16 +536,18 @@ class Gitlab(object):
         if inspect.isclass(obj):
             if not issubclass(obj, GitlabObject):
                 raise GitlabError("Invalid class: %s" % obj)
-            params = {}
-            params[obj.idAttr] = id
-        else:
-            params = obj.__dict__.copy()
+
+        params = {obj.idAttr: id if id else getattr(obj, obj.idAttr)}
         params.update(kwargs)
+
         missing = []
         for k in itertools.chain(obj.requiredUrlAttrs,
                                  obj.requiredDeleteAttrs):
             if k not in params:
-                missing.append(k)
+                try:
+                    params[k] = getattr(obj, k)
+                except KeyError:
+                    missing.append(k)
         if missing:
             raise GitlabDeleteError('Missing attribute(s): %s' %
                                     ", ".join(missing))
@@ -485,13 +560,20 @@ class Gitlab(object):
         # url-parameters left
         for attribute in obj.requiredUrlAttrs:
             del params[attribute]
+        if obj._id_in_delete_url:
+            # The ID is already built, no need to add it as extra key in query
+            # string
+            params.pop(obj.idAttr)
 
         try:
-            r = requests.delete(url,
-                                params=params,
-                                headers=headers,
-                                verify=self.ssl_verify,
-                                timeout=self.timeout)
+            r = self.session.delete(url,
+                                    params=params,
+                                    headers=headers,
+                                    verify=self.ssl_verify,
+                                    timeout=self.timeout,
+                                    auth=requests.auth.HTTPBasicAuth(
+                                        self.http_username,
+                                        self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -528,17 +610,21 @@ class Gitlab(object):
             raise GitlabCreateError('Missing attribute(s): %s' %
                                     ", ".join(missing))
 
-        url = self._construct_url(id_=None, obj=obj, parameters=params)
+        url = self._construct_url(id_=None, obj=obj, parameters=params,
+                                  action='create')
         headers = self._create_headers(content_type="application/json")
 
         # build data that can really be sent to server
         data = obj._data_for_gitlab(extra_parameters=kwargs)
 
         try:
-            r = requests.post(url, data=data,
-                              headers=headers,
-                              verify=self.ssl_verify,
-                              timeout=self.timeout)
+            r = self.session.post(url, data=data,
+                                  headers=headers,
+                                  verify=self.ssl_verify,
+                                  timeout=self.timeout,
+                                  auth=requests.auth.HTTPBasicAuth(
+                                      self.http_username,
+                                      self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
@@ -585,10 +671,13 @@ class Gitlab(object):
         data = obj._data_for_gitlab(extra_parameters=kwargs, update=True)
 
         try:
-            r = requests.put(url, data=data,
-                             headers=headers,
-                             verify=self.ssl_verify,
-                             timeout=self.timeout)
+            r = self.session.put(url, data=data,
+                                 headers=headers,
+                                 verify=self.ssl_verify,
+                                 timeout=self.timeout,
+                                 auth=requests.auth.HTTPBasicAuth(
+                                     self.http_username,
+                                     self.http_password))
         except Exception as e:
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % e)
