@@ -91,6 +91,26 @@ class GitlabUnblockError(GitlabOperationError):
     pass
 
 
+class GitlabSubscribeError(GitlabOperationError):
+    pass
+
+
+class GitlabUnsubscribeError(GitlabOperationError):
+    pass
+
+
+class GitlabMRForbiddenError(GitlabOperationError):
+    pass
+
+
+class GitlabMRClosedError(GitlabOperationError):
+    pass
+
+
+class GitlabMROnBuildSuccessError(GitlabOperationError):
+    pass
+
+
 def raise_error_from_response(response, error, expected_code=200):
     """Tries to parse gitlab error message from response and raises error.
 
@@ -98,20 +118,30 @@ def raise_error_from_response(response, error, expected_code=200):
 
     If response status code is 401, raises instead GitlabAuthenticationError.
 
-    response: requests response object
-    error: Error-class to raise. Should be inherited from GitLabError
+    Args:
+        response: requests response object
+        error: Error-class or dict {return-code => class} of possible error
+               class to raise. Should be inherited from GitLabError
     """
 
-    if expected_code == response.status_code:
+    if isinstance(expected_code, int):
+        expected_codes = [expected_code]
+    else:
+        expected_codes = expected_code
+
+    if response.status_code in expected_codes:
         return
 
     try:
         message = response.json()['message']
-    except (KeyError, ValueError):
+    except (KeyError, ValueError, TypeError):
         message = response.content
 
-    if response.status_code == 401:
-        error = GitlabAuthenticationError
+    if isinstance(error, dict):
+        error = error.get(response.status_code, GitlabOperationError)
+    else:
+        if response.status_code == 401:
+            error = GitlabAuthenticationError
 
     raise error(error_message=message,
                 response_code=response.status_code,
